"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var autoBind = require("auto-bind");
var scalars_1 = require("./scalars");
var utils_1 = require("./utils");
var variables_to_object_1 = require("./variables-to-object");
var naming_1 = require("./naming");
function getRootType(operation, schema) {
    switch (operation) {
        case 'query':
            return schema.getQueryType();
        case 'mutation':
            return schema.getMutationType();
        case 'subscription':
            return schema.getSubscriptionType();
    }
}
var BaseDocumentsVisitor = /** @class */ (function () {
    function BaseDocumentsVisitor(rawConfig, additionalConfig, _schema, defaultScalars) {
        if (defaultScalars === void 0) { defaultScalars = scalars_1.DEFAULT_SCALARS; }
        this._schema = _schema;
        this._declarationBlockConfig = {};
        this._unnamedCounter = 1;
        this._parsedConfig = __assign({ addTypename: !rawConfig.skipTypename, scalars: __assign({}, (defaultScalars || scalars_1.DEFAULT_SCALARS), (rawConfig.scalars || {})), convert: naming_1.convertFactory(rawConfig), typesPrefix: rawConfig.typesPrefix || '' }, (additionalConfig || {}));
        autoBind(this);
        this._variablesTransfomer = new variables_to_object_1.OperationVariablesToObject(this.scalars, this.convertName);
    }
    BaseDocumentsVisitor.prototype.setSelectionSetHandler = function (handler) {
        this._selectionSetToObject = handler;
    };
    BaseDocumentsVisitor.prototype.setDeclarationBlockConfig = function (config) {
        this._declarationBlockConfig = config;
    };
    BaseDocumentsVisitor.prototype.setVariablesTransformer = function (variablesTransfomer) {
        this._variablesTransfomer = variablesTransfomer;
    };
    BaseDocumentsVisitor.prototype.convertName = function (node, options) {
        var useTypesPrefix = options && typeof options.useTypesPrefix === 'boolean' ? options.useTypesPrefix : true;
        return (useTypesPrefix ? this._parsedConfig.typesPrefix : '') + this._parsedConfig.convert(node, options);
    };
    Object.defineProperty(BaseDocumentsVisitor.prototype, "config", {
        get: function () {
            return this._parsedConfig;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseDocumentsVisitor.prototype, "schema", {
        get: function () {
            return this._schema;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseDocumentsVisitor.prototype, "scalars", {
        get: function () {
            return this.config.scalars;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseDocumentsVisitor.prototype, "addTypename", {
        get: function () {
            return this._parsedConfig.addTypename;
        },
        enumerable: true,
        configurable: true
    });
    BaseDocumentsVisitor.prototype.handleAnonymouseOperation = function (node) {
        var name = node.name && node.name.value;
        if (name) {
            return this.convertName(node, {
                useTypesPrefix: false
            });
        }
        return this.convertName(this._unnamedCounter++ + '', {
            prefix: 'Unnamed_',
            suffix: '_',
            useTypesPrefix: false
        });
    };
    BaseDocumentsVisitor.prototype.FragmentDefinition = function (node) {
        var fragmentRootType = this._schema.getType(node.typeCondition.name.value);
        var selectionSet = this._selectionSetToObject.createNext(fragmentRootType, node.selectionSet);
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node, {
            useTypesPrefix: true,
            suffix: 'Fragment'
        }))
            .withContent(selectionSet.string).string;
    };
    BaseDocumentsVisitor.prototype.OperationDefinition = function (node) {
        var name = this.handleAnonymouseOperation(node);
        var operationRootType = getRootType(node.operation, this._schema);
        var selectionSet = this._selectionSetToObject.createNext(operationRootType, node.selectionSet);
        var visitedOperationVariables = this._variablesTransfomer.transform(node.variableDefinitions);
        var operationResult = new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(name, {
            suffix: utils_1.toPascalCase(node.operation)
        }))
            .withContent(selectionSet.string).string;
        var operationVariables = new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(name, {
            suffix: utils_1.toPascalCase(node.operation) + 'Variables'
        }))
            .withBlock(visitedOperationVariables).string;
        return [operationVariables, operationResult].filter(function (r) { return r; }).join('\n\n');
    };
    return BaseDocumentsVisitor;
}());
exports.BaseDocumentsVisitor = BaseDocumentsVisitor;
//# sourceMappingURL=base-documents-visitor.js.map