"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var autoBind = require("auto-bind");
var scalars_1 = require("./scalars");
var utils_1 = require("./utils");
var graphql_1 = require("graphql");
var variables_to_object_1 = require("./variables-to-object");
var naming_1 = require("./naming");
var BaseResolversVisitor = /** @class */ (function () {
    function BaseResolversVisitor(rawConfig, additionalConfig, _schema, defaultScalars) {
        if (defaultScalars === void 0) { defaultScalars = scalars_1.DEFAULT_SCALARS; }
        this._schema = _schema;
        this._declarationBlockConfig = {};
        this._collectedResolvers = {};
        this._collectedDirectiveResolvers = {};
        this._parsedConfig = __assign({ scalars: __assign({}, (defaultScalars || scalars_1.DEFAULT_SCALARS), (rawConfig.scalars || {})), convert: naming_1.convertFactory(rawConfig), typesPrefix: rawConfig.typesPrefix || '', contextType: rawConfig.contextType || 'any', mappers: this.transformMappers(rawConfig.mappers || {}) }, (additionalConfig || {}));
        autoBind(this);
        this._variablesTransfomer = new variables_to_object_1.OperationVariablesToObject(this.scalars, this.convertName);
    }
    BaseResolversVisitor.prototype.isExternalMapper = function (value) {
        return value.includes('#');
    };
    BaseResolversVisitor.prototype.parseMapper = function (mapper) {
        if (this.isExternalMapper(mapper)) {
            var _a = mapper.split('#'), source = _a[0], type = _a[1];
            return {
                isExternal: true,
                source: source,
                type: type
            };
        }
        return {
            isExternal: false,
            type: mapper
        };
    };
    BaseResolversVisitor.prototype.transformMappers = function (rawMappers) {
        var _this = this;
        var result = {};
        Object.keys(rawMappers).forEach(function (gqlTypeName) {
            var mapperDef = rawMappers[gqlTypeName];
            var parsedMapper = _this.parseMapper(mapperDef);
            result[gqlTypeName] = parsedMapper;
        });
        return result;
    };
    Object.defineProperty(BaseResolversVisitor.prototype, "config", {
        get: function () {
            return this._parsedConfig;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseResolversVisitor.prototype, "schema", {
        get: function () {
            return this._schema;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseResolversVisitor.prototype, "scalars", {
        get: function () {
            return this.config.scalars;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseResolversVisitor.prototype, "mappersImports", {
        get: function () {
            var _this = this;
            var groupedMappers = {};
            Object.keys(this.config.mappers)
                .filter(function (gqlTypeName) { return _this.config.mappers[gqlTypeName].isExternal; })
                .forEach(function (gqlTypeName) {
                var mapper = _this.config.mappers[gqlTypeName];
                if (!groupedMappers[mapper.source]) {
                    groupedMappers[mapper.source] = [];
                }
                groupedMappers[mapper.source].push(mapper.type);
            });
            return Object.keys(groupedMappers).map(function (source) { return _this.buildMapperImport(source, groupedMappers[source]); });
        },
        enumerable: true,
        configurable: true
    });
    BaseResolversVisitor.prototype.buildMapperImport = function (source, types) {
        return "import { " + types.join(', ') + " } from '" + source + "';";
    };
    BaseResolversVisitor.prototype.convertName = function (name, options) {
        var useTypesPrefix = options && typeof options.useTypesPrefix === 'boolean' ? options.useTypesPrefix : true;
        return (useTypesPrefix ? this.config.typesPrefix : '') + this.config.convert(name, options);
    };
    BaseResolversVisitor.prototype.setDeclarationBlockConfig = function (config) {
        this._declarationBlockConfig = config;
    };
    BaseResolversVisitor.prototype.setVariablesTransformer = function (variablesTransfomer) {
        this._variablesTransfomer = variablesTransfomer;
    };
    BaseResolversVisitor.prototype.getRootResolver = function () {
        var _this = this;
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName('IResolvers'), "<Context = " + this.config.contextType + ">")
            .withBlock(Object.keys(this._collectedResolvers)
            .map(function (schemaTypeName) {
            var resolverType = _this._collectedResolvers[schemaTypeName];
            return utils_1.indent(_this.formatRootResolver(schemaTypeName, resolverType));
        })
            .join('\n')).string;
    };
    BaseResolversVisitor.prototype.formatRootResolver = function (schemaTypeName, resolverType) {
        return schemaTypeName + "?: " + resolverType + ",";
    };
    BaseResolversVisitor.prototype.getAllDirectiveResolvers = function () {
        var _this = this;
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName('IDirectiveResolvers'), "<Context = " + this.config.contextType + ">")
            .withBlock(Object.keys(this._collectedDirectiveResolvers)
            .map(function (schemaTypeName) {
            var resolverType = _this._collectedDirectiveResolvers[schemaTypeName];
            return utils_1.indent(_this.formatRootResolver(schemaTypeName, resolverType));
        })
            .join('\n')).string;
    };
    BaseResolversVisitor.prototype.Name = function (node) {
        return node.value;
    };
    BaseResolversVisitor.prototype.ListType = function (node) {
        var asString = node.type;
        return "Array<" + asString + ">";
    };
    BaseResolversVisitor.prototype.NamedType = function (node) {
        var type = this.config.scalars[node.name] || this.convertName(node);
        return "" + type;
    };
    BaseResolversVisitor.prototype.NonNullType = function (node) {
        var asString = node.type;
        return asString;
    };
    BaseResolversVisitor.prototype.FieldDefinition = function (node, key, parent) {
        var _this = this;
        var hasArguments = node.arguments && node.arguments.length > 0;
        return function (parentName) {
            var original = parent[key];
            var realType = utils_1.getBaseTypeNode(original.type).name.value;
            var mappedType = _this.config.mappers[realType]
                ? _this._variablesTransfomer.wrapAstTypeWithModifiers(_this.config.mappers[realType].type, original.type)
                : node.type;
            var subscriptionType = _this._schema.getSubscriptionType();
            var isSubscriptionType = subscriptionType && subscriptionType.name === parentName;
            return utils_1.indent(node.name + "?: " + (isSubscriptionType ? 'SubscriptionResolver' : 'Resolver') + "<" + mappedType + ", ParentType, Context" + (hasArguments
                ? ", " + (_this.convertName(parentName, {
                    useTypesPrefix: true
                }) +
                    _this.convertName(node.name, {
                        useTypesPrefix: false
                    }) +
                    'Args')
                : '') + ">,");
        };
    };
    BaseResolversVisitor.prototype.ObjectTypeDefinition = function (node) {
        var name = this.convertName(node, {
            suffix: 'Resolvers'
        });
        var type = null;
        if (this.config.mappers[node.name]) {
            type = this.config.mappers[node.name].type;
        }
        else {
            type = this.config.scalars[node.name] || this.convertName(node);
        }
        var block = new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('interface')
            .withName(name, "<Context = " + this.config.contextType + ", ParentType = " + type + ">")
            .withBlock(node.fields.map(function (f) { return f(node.name); }).join('\n'));
        this._collectedResolvers[node.name] = name + '<Context>';
        return block.string;
    };
    BaseResolversVisitor.prototype.UnionTypeDefinition = function (node, key, parent) {
        var _this = this;
        var name = this.convertName(node, {
            suffix: 'Resolvers'
        });
        var originalNode = parent[key];
        var possibleTypes = originalNode.types
            .map(function (node) { return _this.convertName(node); })
            .map(function (f) { return "'" + f + "'"; })
            .join(' | ');
        this._collectedResolvers[node.name] = name;
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('interface')
            .withName(name, "<Context = " + this.config.contextType + ", ParentType = " + node.name + ">")
            .withBlock(utils_1.indent("__resolveType: TypeResolveFn<" + possibleTypes + ">")).string;
    };
    BaseResolversVisitor.prototype.ScalarTypeDefinition = function (node) {
        var baseName = this.convertName(node);
        this._collectedResolvers[node.name] = 'GraphQLScalarType';
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('interface')
            .withName(this.convertName(node, {
            suffix: 'ScalarConfig'
        }), " extends GraphQLScalarTypeConfig<" + baseName + ", any>")
            .withBlock(utils_1.indent("name: '" + node.name + "'")).string;
    };
    BaseResolversVisitor.prototype.DirectiveDefinition = function (node) {
        var directiveName = this.convertName(node, {
            suffix: 'DirectiveResolver'
        });
        var hasArguments = node.arguments && node.arguments.length > 0;
        var directiveArgs = hasArguments
            ? this._variablesTransfomer.transform(node.arguments)
            : '';
        this._collectedDirectiveResolvers[node.name] = directiveName + '<any, any, Context>';
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(directiveName, "<Result, Parent, Context = " + this.config.contextType + ", Args = { " + directiveArgs + " }>")
            .withContent("DirectiveResolverFn<Result, Parent, Context, Args>").string;
    };
    BaseResolversVisitor.prototype.InterfaceTypeDefinition = function (node) {
        var name = this.convertName(node, {
            suffix: 'Resolvers'
        });
        var allTypesMap = this._schema.getTypeMap();
        var implementingTypes = [];
        this._collectedResolvers[node.name] = name;
        for (var _i = 0, _a = Object.values(allTypesMap); _i < _a.length; _i++) {
            var graphqlType = _a[_i];
            if (graphqlType instanceof graphql_1.GraphQLObjectType) {
                var allInterfaces = graphqlType.getInterfaces();
                if (allInterfaces.find(function (int) { return int.name === node.name; })) {
                    implementingTypes.push(graphqlType.name);
                }
            }
        }
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('interface')
            .withName(name, "<Context = " + this.config.contextType + ", ParentType = " + node.name + ">")
            .withBlock(utils_1.indent("__resolveType: TypeResolveFn<" + implementingTypes.map(function (name) { return "'" + name + "'"; }).join(' | ') + ">"))
            .string;
    };
    return BaseResolversVisitor;
}());
exports.BaseResolversVisitor = BaseResolversVisitor;
//# sourceMappingURL=base-resolvers-visitor.js.map