"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var base_visitor_1 = require("./base-visitor");
var variables_to_object_1 = require("./variables-to-object");
var utils_1 = require("./utils");
var scalars_1 = require("./scalars");
var BaseTypesVisitor = /** @class */ (function (_super) {
    __extends(BaseTypesVisitor, _super);
    function BaseTypesVisitor(rawConfig, additionalConfig, defaultScalars) {
        if (defaultScalars === void 0) { defaultScalars = scalars_1.DEFAULT_SCALARS; }
        var _this = _super.call(this, rawConfig, __assign({ enumValues: rawConfig.enumValues || {} }, additionalConfig), defaultScalars) || this;
        _this._argumentsTransformer = new variables_to_object_1.OperationVariablesToObject(_this.scalars, _this.convertName);
        return _this;
    }
    BaseTypesVisitor.prototype.setDeclarationBlockConfig = function (config) {
        this._declarationBlockConfig = config;
    };
    BaseTypesVisitor.prototype.setArgumentsTransformer = function (argumentsTransfomer) {
        this._argumentsTransformer = argumentsTransfomer;
    };
    BaseTypesVisitor.prototype.NonNullType = function (node) {
        var asString = node.type;
        return asString;
    };
    BaseTypesVisitor.prototype.InputObjectTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node))
            .withBlock(node.fields.join('\n')).string;
    };
    BaseTypesVisitor.prototype.InputValueDefinition = function (node) {
        return utils_1.indent(node.name + ": " + node.type + ",");
    };
    BaseTypesVisitor.prototype.Name = function (node) {
        return node.value;
    };
    BaseTypesVisitor.prototype.FieldDefinition = function (node) {
        var typeString = node.type;
        return utils_1.indent(node.name + ": " + typeString + ",");
    };
    BaseTypesVisitor.prototype.UnionTypeDefinition = function (node, key, parent) {
        var _this = this;
        var originalNode = parent[key];
        var possibleTypes = originalNode.types.map(function (t) { return _this.convertName(t); }).join(' | ');
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node))
            .withContent(possibleTypes).string;
    };
    BaseTypesVisitor.prototype.ObjectTypeDefinition = function (node, key, parent) {
        var _this = this;
        var originalNode = parent[key];
        var interfaces = originalNode.interfaces && node.interfaces.length > 0
            ? originalNode.interfaces.map(function (i) { return _this.convertName(i); }).join(' & ') + ' & '
            : '';
        var typeDefinition = new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node))
            .withContent(interfaces)
            .withBlock(node.fields.join('\n')).string;
        var original = parent[key];
        var fieldsWithArguments = original.fields.filter(function (field) { return field.arguments && field.arguments.length > 0; });
        var fieldsArguments = fieldsWithArguments.map(function (field) {
            var name = original.name.value +
                _this.convertName(field, {
                    useTypesPrefix: false
                }) +
                'Args';
            return new utils_1.DeclarationBlock(_this._declarationBlockConfig)
                .export()
                .asKind('type')
                .withName(_this.convertName(name))
                .withBlock(_this._argumentsTransformer.transform(field.arguments)).string;
        });
        return [typeDefinition].concat(fieldsArguments).filter(function (f) { return f; }).join('\n\n');
    };
    BaseTypesVisitor.prototype.InterfaceTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node))
            .withBlock(node.fields.join('\n')).string;
    };
    BaseTypesVisitor.prototype.ScalarTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node))
            .withContent(this.config.scalars[node.name] || 'any').string;
    };
    BaseTypesVisitor.prototype.EnumTypeDefinition = function (node) {
        return new utils_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('enum')
            .withName(this.convertName(node))
            .withBlock(this.buildEnumValuesBlock(node.values)).string;
    };
    BaseTypesVisitor.prototype.buildEnumValuesBlock = function (values) {
        var _this = this;
        return values
            .map(function (enumOption) {
            return utils_1.indent("" + _this.convertName(enumOption) + _this._declarationBlockConfig.enumNameValueSeparator + " " + utils_1.wrapWithSingleQuotes(_this.config.enumValues[enumOption.name] || enumOption.name));
        })
            .join(', \n');
    };
    BaseTypesVisitor.prototype.DirectiveDefinition = function (node) {
        return '';
    };
    BaseTypesVisitor.prototype.NamedType = function (node) {
        var type = this.scalars[node.name] || this.convertName(node);
        return type;
    };
    BaseTypesVisitor.prototype.ListType = function (node) {
        var asString = node.type;
        return this.wrapWithListType(asString);
    };
    BaseTypesVisitor.prototype.wrapWithListType = function (str) {
        return "Array<" + str + ">";
    };
    return BaseTypesVisitor;
}(base_visitor_1.BaseVisitor));
exports.BaseTypesVisitor = BaseTypesVisitor;
//# sourceMappingURL=base-types-visitor.js.map