"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("./index");
var autoBind = require("auto-bind");
var graphql_1 = require("graphql");
var dependency_graph_1 = require("dependency-graph");
var graphql_tag_1 = require("graphql-tag");
var graphql_codegen_core_1 = require("graphql-codegen-core");
var utils_1 = require("./utils");
var ClientSideBaseVisitor = /** @class */ (function (_super) {
    __extends(ClientSideBaseVisitor, _super);
    function ClientSideBaseVisitor(_fragments, rawConfig, additionalConfig) {
        var _this = _super.call(this, rawConfig, __assign({ noGraphQLTag: utils_1.getConfigValue(rawConfig.noGraphQLTag, false), gqlImport: rawConfig.gqlImport || null }, additionalConfig)) || this;
        _this._fragments = _fragments;
        autoBind(_this);
        return _this;
    }
    ClientSideBaseVisitor.prototype._getFragmentName = function (fragment) {
        return (typeof fragment === 'string' ? fragment : fragment.name.value) + 'FragmentDoc';
    };
    ClientSideBaseVisitor.prototype._extractFragments = function (document) {
        return (graphql_1.print(document).match(/\.\.\.[a-z0-9\_]+/gi) || []).map(function (name) { return name.replace('...', ''); });
    };
    ClientSideBaseVisitor.prototype._transformFragments = function (document) {
        var _this = this;
        return this._extractFragments(document).map(function (document) { return _this._getFragmentName(document); });
    };
    ClientSideBaseVisitor.prototype._includeFragments = function (fragments) {
        if (fragments) {
            return "" + fragments
                .filter(function (name, i, all) { return all.indexOf(name) === i; })
                .map(function (name) { return '${' + name + '}'; })
                .join('\n');
        }
        return '';
    };
    ClientSideBaseVisitor.prototype._prepareDocument = function (documentStr) {
        return documentStr;
    };
    ClientSideBaseVisitor.prototype._gql = function (node) {
        var doc = this._prepareDocument("\n    " + graphql_1.print(node) + "\n    " + this._includeFragments(this._transformFragments(node)));
        if (this.config.noGraphQLTag) {
            return JSON.stringify(graphql_tag_1.default(doc));
        }
        return 'gql`' + doc + '`';
    };
    ClientSideBaseVisitor.prototype._generateFragment = function (fragmentDocument) {
        var name = this._getFragmentName(fragmentDocument);
        return "export const " + name + (this.config.noGraphQLTag ? ': DocumentNode' : '') + " = " + this._gql(fragmentDocument) + ";";
    };
    Object.defineProperty(ClientSideBaseVisitor.prototype, "fragments", {
        get: function () {
            var _this = this;
            if (this._fragments.length === 0) {
                return '';
            }
            var graph = new dependency_graph_1.DepGraph({ circular: true });
            for (var _i = 0, _a = this._fragments; _i < _a.length; _i++) {
                var fragment = _a[_i];
                if (graph.hasNode(fragment.name.value)) {
                    var cachedAsString = graphql_1.print(graph.getNodeData(fragment.name.value));
                    var asString = graphql_1.print(fragment);
                    if (cachedAsString !== asString) {
                        throw new Error("Duplicated fragment called '" + fragment.name + "'!");
                    }
                }
                graph.addNode(fragment.name.value, fragment);
            }
            this._fragments.forEach(function (fragment) {
                var depends = _this._extractFragments(fragment);
                if (depends) {
                    depends.forEach(function (name) {
                        graph.addDependency(fragment.name.value, name);
                    });
                }
            });
            return graph
                .overallOrder()
                .map(function (name) { return _this._generateFragment(graph.getNodeData(name)); })
                .join('\n');
        },
        enumerable: true,
        configurable: true
    });
    ClientSideBaseVisitor.prototype._parseImport = function (importStr) {
        var _a = importStr.split('#'), moduleName = _a[0], propName = _a[1];
        return {
            moduleName: moduleName,
            propName: propName
        };
    };
    ClientSideBaseVisitor.prototype.getImports = function () {
        var gqlImport = this._parseImport(this.config.gqlImport || 'graphql-tag');
        var imports = [];
        if (!this.config.noGraphQLTag) {
            imports.push("\nimport " + (gqlImport.propName ? "{ " + (gqlImport.propName === 'gql' ? 'gql' : gqlImport.propName + " as gql") + " }" : 'gql') + " from '" + gqlImport.moduleName + "';");
        }
        else {
            imports.push("import { DocumentNode } from 'graphql';");
        }
        return imports.join('\n');
    };
    ClientSideBaseVisitor.prototype.buildOperation = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        return null;
    };
    ClientSideBaseVisitor.prototype.OperationDefinition = function (node) {
        if (!node.name || !node.name.value) {
            return null;
        }
        var documentVariableName = this.convertName(node, {
            suffix: 'Document'
        });
        var documentString = "export const " + documentVariableName + (this.config.noGraphQLTag ? ': DocumentNode' : '') + " = " + this._gql(node) + ";";
        var operationType = graphql_codegen_core_1.toPascalCase(node.operation);
        var operationResultType = this.convertName(node, {
            suffix: operationType
        });
        var operationVariablesTypes = this.convertName(node, {
            suffix: operationType + 'Variables'
        });
        var additional = this.buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
        return [documentString, additional].filter(function (a) { return a; }).join('\n');
    };
    return ClientSideBaseVisitor;
}(index_1.BaseVisitor));
exports.ClientSideBaseVisitor = ClientSideBaseVisitor;
//# sourceMappingURL=client-side-base-visitor.js.map