"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var change_case_1 = require("change-case");
var graphql_1 = require("graphql");
function isWrapperType(t) {
    return graphql_1.isListType(t) || graphql_1.isNonNullType(t);
}
exports.getConfigValue = function (value, defaultValue) {
    if (value === null || value === undefined) {
        return defaultValue;
    }
    return value;
};
function getBaseType(type) {
    if (isWrapperType(type)) {
        return getBaseType(type.ofType);
    }
    else {
        return type;
    }
}
exports.getBaseType = getBaseType;
function quoteIfNeeded(array, joinWith) {
    if (joinWith === void 0) { joinWith = ' & '; }
    if (array.length === 0) {
        return '';
    }
    else if (array.length === 1) {
        return array[0];
    }
    else {
        return "(" + array.join(joinWith) + ")";
    }
}
exports.quoteIfNeeded = quoteIfNeeded;
function block(array) {
    return array && array.length !== 0 ? '{\n' + array.join('\n') + '\n}' : '';
}
exports.block = block;
function wrapWithSingleQuotes(str) {
    return "'" + str + "'";
}
exports.wrapWithSingleQuotes = wrapWithSingleQuotes;
function breakLine(str) {
    return str + '\n';
}
exports.breakLine = breakLine;
function indent(str) {
    return '  ' + str;
}
exports.indent = indent;
var DeclarationBlock = /** @class */ (function () {
    function DeclarationBlock(_config) {
        this._config = _config;
        this._export = false;
        this._name = null;
        this._kind = null;
        this._methodName = null;
        this._content = null;
        this._block = null;
        this._nameGenerics = null;
        this._config = __assign({ blockWrapper: '', enumNameValueSeparator: ':' }, this._config);
    }
    DeclarationBlock.prototype.export = function (exp) {
        if (exp === void 0) { exp = true; }
        this._export = exp;
        return this;
    };
    DeclarationBlock.prototype.asKind = function (kind) {
        this._kind = kind;
        return this;
    };
    DeclarationBlock.prototype.withMethodCall = function (methodName) {
        this._methodName = methodName;
        return this;
    };
    DeclarationBlock.prototype.withBlock = function (block) {
        this._block = block;
        return this;
    };
    DeclarationBlock.prototype.withContent = function (content) {
        this._content = content;
        return this;
    };
    DeclarationBlock.prototype.withName = function (name, generics) {
        if (generics === void 0) { generics = null; }
        this._name = name;
        this._nameGenerics = generics;
        return this;
    };
    Object.defineProperty(DeclarationBlock.prototype, "string", {
        get: function () {
            var result = '';
            if (this._export) {
                result += 'export ';
            }
            if (this._kind) {
                var extra = '';
                var name = '';
                if (['type', 'const', 'var', 'let'].includes(this._kind)) {
                    extra = '= ';
                }
                if (this._name) {
                    name = this._name + (this._nameGenerics || '') + ' ';
                }
                result += this._kind + ' ' + name + extra;
            }
            if (this._block) {
                if (this._content) {
                    result += this._content;
                }
                if (this._methodName) {
                    result += this._methodName + "({" + this._config.blockWrapper + "\n" + this._block + "\n" + this._config.blockWrapper + "})";
                }
                else {
                    result += "{" + this._config.blockWrapper + "\n" + this._block + "\n" + this._config.blockWrapper + "}";
                }
            }
            else if (this._content) {
                result += this._content;
            }
            else if (this._kind) {
                result += '{}';
            }
            return result + (this._kind === 'interface' ? '' : ';') + '\n';
        },
        enumerable: true,
        configurable: true
    });
    return DeclarationBlock;
}());
exports.DeclarationBlock = DeclarationBlock;
function getBaseTypeNode(typeNode) {
    if (typeNode.kind === graphql_1.Kind.LIST_TYPE || typeNode.kind === graphql_1.Kind.NON_NULL_TYPE) {
        return getBaseTypeNode(typeNode.type);
    }
    return typeNode;
}
exports.getBaseTypeNode = getBaseTypeNode;
function toPascalCase(str) {
    return str
        .split('_')
        .map(function (s) { return change_case_1.pascalCase(s); })
        .join('_');
}
exports.toPascalCase = toPascalCase;
exports.wrapTypeWithModifiers = function (prefix) {
    if (prefix === void 0) { prefix = ''; }
    return function (baseType, type) {
        if (graphql_1.isNonNullType(type)) {
            return exports.wrapTypeWithModifiers(prefix)(baseType, type.ofType).substr(1);
        }
        else if (graphql_1.isListType(type)) {
            var innerType = exports.wrapTypeWithModifiers(prefix)(baseType, type.ofType);
            return prefix + "Array<" + innerType + ">";
        }
        else {
            return "" + prefix + baseType;
        }
    };
};
//# sourceMappingURL=utils.js.map