"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var utils_1 = require("./utils");
var autoBind = require("auto-bind");
var OperationVariablesToObject = /** @class */ (function () {
    function OperationVariablesToObject(_scalars, _convertName) {
        this._scalars = _scalars;
        this._convertName = _convertName;
        autoBind(this);
    }
    OperationVariablesToObject.prototype.getName = function (node) {
        if (node.name) {
            if (typeof node.name === 'string') {
                return node.name;
            }
            return node.name.value;
        }
        else if (node.variable) {
            return node.variable.name.value;
        }
        return null;
    };
    OperationVariablesToObject.prototype.transform = function (variablesNode) {
        var _this = this;
        if (!variablesNode || variablesNode.length === 0) {
            return null;
        }
        return variablesNode.map(function (variable) { return utils_1.indent(_this.transformVariable(variable)); }).join(',\n');
    };
    OperationVariablesToObject.prototype.transformVariable = function (variable) {
        var baseType = typeof variable.type === 'string' ? variable.type : utils_1.getBaseTypeNode(variable.type);
        var typeName = typeof baseType === 'string' ? baseType : baseType.name.value;
        var typeValue = this._scalars[typeName] ||
            this._convertName(baseType, {
                useTypesPrefix: true
            });
        var fieldName = this.getName(variable);
        var fieldType = this.wrapAstTypeWithModifiers(typeValue, variable.type);
        var hasDefaultValue = variable.defaultValue != null && typeof variable.defaultValue !== 'undefined';
        var isNonNullType = variable.type.kind === graphql_1.Kind.NON_NULL_TYPE;
        var formattedFieldString = this.formatFieldString(fieldName, isNonNullType, hasDefaultValue);
        var formattedTypeString = this.formatTypeString(fieldType, isNonNullType, hasDefaultValue);
        return formattedFieldString + ": " + formattedTypeString;
    };
    OperationVariablesToObject.prototype.wrapAstTypeWithModifiers = function (baseType, typeNode) {
        throw new Error("You must override \"wrapAstTypeWithModifiers\" of OperationVariablesToObject!");
    };
    OperationVariablesToObject.prototype.formatFieldString = function (fieldName, isNonNullType, hasDefaultValue) {
        return fieldName;
    };
    OperationVariablesToObject.prototype.formatTypeString = function (fieldType, isNonNullType, hasDefaultValue) {
        return fieldType;
    };
    return OperationVariablesToObject;
}());
exports.OperationVariablesToObject = OperationVariablesToObject;
//# sourceMappingURL=variables-to-object.js.map